#include <gccore.h>
#include <ogcsys.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gbcore/gb.h"
#include "sound.h"
#include "filer.h"
#include "screen.h"
#include "fatfs/tff.h"

char oldpath[MAX_PATH]={0};

int load_rom()
{
	FIL fp;
	FILINFO finfo;
	WORD bytes_read;
	
	char RomPath[MAX_PATH];
	int romsize, ramsize;
	
	strcpy(RomPath, oldpath);
	getFilePath(RomPath,EXT_GB);
	
	if(f_stat(RomPath, &finfo) != FR_OK)
		return 0;
	if(finfo.fsize > MAX_ROM_SIZE)
		return 0;
	if(f_open(&fp, RomPath, FA_READ) != FR_OK)
		return 0;
	
	romsize = 0;
	
	while(romsize < finfo.fsize)
	{
		if(f_read(&fp, rom_image + romsize, 0x200, &bytes_read) != FR_OK){
			f_close(&fp);
			return 0;
		}
		if(bytes_read == 0)
			break;
		romsize += bytes_read;
	}
	f_close(&fp);
	
	if(romsize < finfo.fsize){
		return 0;
	}
	
	ramsize = 0;
	memset(sram_space, 0, sizeof(sram_space));
	
	if (!gb_load_rom(rom_image, romsize, sram_space, ramsize))
		return 0;
	
	strcpy(oldpath, RomPath);
	
	return 1;
}

void mainloop(void)
{
	int line;
	
	wavoutStart();
	for(;;) {
		for(line=0; line<154; line++)
			gb_run();
		
		VIDEO_SetNextFramebuffer(xfb[pg_drawframe]);
		VIDEO_Flush();
		VIDEO_WaitVSync();
		pg_showframe ^= 1;
		pg_drawframe ^= 1;
		
		if(bMenu){
			wavoutStop();
			gb_init();
			while(!load_rom())
				;
			wavoutStart();
			bMenu = 0;
		}
	}
}

int main ()
{
	PAD_Init();
	screenInit();
	wavoutInit();
	
	FATFS ffs;
	if(f_mount(0, &ffs) != FR_OK)
		return 0;
	
	gb_init();
	while(!load_rom())
		;
	
	mainloop();
	
	return 0;
}
