#include <gccore.h>
#include <ogcsys.h>
#include "fontNaga10.h"

GXRModeObj *vmode;
u32 *xfb[2] = { NULL, NULL };

long pg_showframe;
long pg_drawframe;

void screenInit()
{
	VIDEO_Init();
	
	switch (VIDEO_GetCurrentTvMode ()){
	case VI_NTSC:
		vmode = &TVNtsc480IntDf;
		break;
	case VI_PAL:
		vmode = &TVPal528IntDf;
		break;
	case VI_MPAL:
		vmode = &TVMpal480IntDf;
		break;
	default:
		vmode = &TVNtsc480IntDf;
		break;
    }

	VIDEO_Configure (vmode);

	pg_showframe = 0;
	pg_drawframe = 1;

	xfb[0] = (u32 *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));
	xfb[1] = (u32 *) MEM_K0_TO_K1 (SYS_AllocateFramebuffer (vmode));
	
	console_init (xfb[0], 20, 64, vmode->fbWidth, vmode->xfbHeight, vmode->fbWidth * 2);
	
	VIDEO_ClearFrameBuffer (vmode, xfb[0], COLOR_BLACK);
	VIDEO_ClearFrameBuffer (vmode, xfb[1], COLOR_BLACK);

	VIDEO_Configure (vmode);
	VIDEO_SetNextFramebuffer (xfb[0]);
	VIDEO_SetBlack(0);
	VIDEO_Flush ();
	VIDEO_WaitVSync ();
	if (vmode->viTVMode & VI_NON_INTERLACE)
		VIDEO_WaitVSync ();
}

// by kwn
void Draw_Char_Hankaku(int x,int y,const unsigned char c,u32 col)
{
	unsigned char  *fnt;
	unsigned char  pt;
	unsigned char ch;
	int x1,y1;

	ch = c;

	// mapping
	if (ch<0x20)
		ch = 0;
	else if (ch<0x80)
		ch -= 0x20;
	else if (ch<0xa0)
		ch = 0;
	else
		ch -= 0x40;

	fnt = (unsigned char *)&hankaku_font10[ch*10];

	// draw
	u32 *dst = &xfb[pg_drawframe][y*320+x/2];
	for(y1=0;y1<10;y1++) {
		pt = *fnt++;
		for(x1=0;x1<5;x1++) {
			if (pt & 1)
				*(dst+320) = *dst = col;
			dst++;
			pt = pt >> 1;
		}
		dst = dst+640-5;
	}
}

// by kwn
void Draw_Char_Zenkaku(int x,int y,const unsigned char u,unsigned char d,u32 col)
{
	// ELISA100.FNTɑ݂Ȃ
	const unsigned short font404[] = {
		0xA2AF, 11,
		0xA2C2, 8,
		0xA2D1, 11,
		0xA2EB, 7,
		0xA2FA, 4,
		0xA3A1, 15,
		0xA3BA, 7,
		0xA3DB, 6,
		0xA3FB, 4,
		0xA4F4, 11,
		0xA5F7, 8,
		0xA6B9, 8,
		0xA6D9, 38,
		0xA7C2, 15,
		0xA7F2, 13,
		0xA8C1, 720,
		0xCFD4, 43,
		0xF4A5, 1030,
		0,0
	};
	unsigned short *fnt;
	unsigned short pt;
	int x1,y1;

	unsigned long n;
	unsigned short code;
	int i;

	// SJISR[h̐
	code = u;
	code = (code<<8) + d;

	// SJISEUCɕϊ
	if(code >= 0xE000) code-=0x4000;
	code = ((((code>>8)&0xFF)-0x81)<<9) + (code&0x00FF);
	if((code & 0x00FF) >= 0x80) code--;
	if((code & 0x00FF) >= 0x9E) code+=0x62;
	else code-=0x40;
	code += 0x2121 + 0x8080;

	// EUCbtHg̔ԍ𐶐
	n = (((code>>8)&0xFF)-0xA1)*(0xFF-0xA1)
		+ (code&0xFF)-0xA1;
	i=0;
	while(font404[i]) {
		if(code >= font404[i]) {
			if(code <= font404[i]+font404[i+1]-1) {
				n = -1;
				break;
			} else {
				n-=font404[i+1];
			}
		}
		i+=2;
	}
	fnt = (unsigned short *)&zenkaku_font10[n*10];

	// draw
	u32 *dst = &xfb[pg_drawframe][y*320+x/2];
	for(y1=0;y1<10;y1++) {
		pt = *fnt++;
		for(x1=0;x1<10;x1++) {
			if (pt & 1)
				*(dst+320) = *dst = col;
			dst++;
			pt = pt >> 1;
		}
		dst = dst+640-10;
	}
}

// by kwn
void mh_print(int x,int y,const char *msg,u32 col)
{
	int xx = x;
	unsigned char ch = 0,bef = 0, *str=(unsigned char*)msg;

	while(*str != 0) {
		ch = *str++;
		if (bef!=0) {
			Draw_Char_Zenkaku(x,y,bef,ch,col);
			x+=20;
			bef=0;
		} else {
			if (((ch>=0x80) && (ch<0xa0)) || (ch>=0xe0)) {
				bef = ch;
			} else {
				if (ch=='\n'){
					x=xx;
					y+=10;
				}else{
					Draw_Char_Hankaku(x,y,ch,col);
					x+=10;
				}
			}
		}
		if (x>=480) break;
	}
}
